<?php
namespace Ak\Hydrator;

use Laminas\Hydrator\AbstractHydrator;
use Laminas\Hydrator\ExtractionInterface;
use Laminas\Hydrator\HydrationInterface;
use Laminas\Hydrator\NamingStrategy\NamingStrategyInterface;
use Laminas\Hydrator\NamingStrategyEnabledInterface;
use Laminas\Hydrator\Strategy;
use Laminas\Hydrator\StrategyEnabledInterface;

/**
 * Omogućava korištenje odvojenih hidratora za ekstrakciju i hidriranje.
 *
 * @package Ak\Hydrator
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/hydrator
 */
class CompositeHydrator extends AbstractHydrator
{
    /** @var HydrationInterface */
    private $hydrator;
    /** @var ExtractionInterface */
    private $extractor;

    /**
     * Konstruktor.
     *
     * @param HydrationInterface $hydrator
     * @param ExtractionInterface $extractor
     */
    public function __construct(HydrationInterface $hydrator = null, ExtractionInterface $extractor = null)
    {
        parent::__construct();

        $this->hydrator = $hydrator;
        $this->extractor = $extractor;
    }

    /**
     * Vraća strategiju za dato ime.
     *
     * @param string $name
     *
     * @return \Laminas\Hydrator\Strategy\StrategyInterface
     */
    public function getStrategy($name)
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof StrategyEnabledInterface) {
                return $hydrator->getStrategy($name);
            }
        }

        return parent::getStrategy($name);
    }

    /**
     * Da li postoji strategija pod datim imenom?
     *
     * @param string $name
     *
     * @return bool
     */
    public function hasStrategy($name)
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof StrategyEnabledInterface) {
                return $hydrator->hasStrategy($name);
            }
        }

        return parent::hasStrategy($name);
    }

    /**
     * Postavlja datu strategiju za dato ime.
     *
     * @param string $name
     * @param Strategy\StrategyInterface $strategy
     *
     * @return $this
     */
    public function addStrategy($name, Strategy\StrategyInterface $strategy)
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof StrategyEnabledInterface) {
                $hydrator->addStrategy($name, $strategy);
            }
        }

        return $this;
    }

    /**
     * Uklanja strategiju registriranu pod datim imenom.
     *
     * @param string $name
     *
     * @return $this
     */
    public function removeStrategy($name)
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof StrategyEnabledInterface) {
                $hydrator->removeStrategy($name);
            }
        }

        return $this;
    }

    /**
     * @return NamingStrategyInterface
     */
    public function getNamingStrategy()
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof NamingStrategyEnabledInterface) {
                return $hydrator->getNamingStrategy();
            }
        }

        return parent::getNamingStrategy();
    }

    /**
     * @return bool
     */
    public function hasNamingStrategy()
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof NamingStrategyEnabledInterface && $hydrator->hasNamingStrategy()) {
                return true;
            }
        }

        return parent::hasNamingStrategy();
    }

    /**
     * @param NamingStrategyInterface $strategy
     * @return AbstractHydrator
     */
    public function setNamingStrategy(NamingStrategyInterface $strategy)
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof NamingStrategyEnabledInterface) {
                $hydrator->setNamingStrategy($strategy);
            }
        }

        return $this;
    }

    /**
     * @return AbstractHydrator
     */
    public function removeNamingStrategy()
    {
        foreach ([$this->extractor, $this->hydrator] as $hydrator) {
            if ($hydrator instanceof NamingStrategyEnabledInterface) {
                $hydrator->removeNamingStrategy();
            }
        }

        return $this;
    }

    /**
     * Ektraktira podatke iz datog objekta.
     *
     * @param  object $object
     *
     * @return array
     */
    public function extract($object)
    {
        return $this->extractor->extract($object);
    }

    /**
     * Hidrira objekt s datim podacima.
     *
     * @param  array $data
     * @param  object $object
     *
     * @return object
     */
    public function hydrate(array $data, $object)
    {
        return $this->hydrator->hydrate($data, $object);
    }
}
